/// <reference types="node" />
import type { IncomingHttpHeaders } from "http";
import type { NextApiRequest, NextApiResponse } from "next";
import type { Params, Routes, MethodsByRoute, RequestBody, ResponseBody, RequestHeaders, ResponseHeaders } from "../index.js";
export { ServerError } from "./error.js";
declare type Handler<R extends Routes> = (req: NextApiRequest, res: NextApiResponse<ResponseBody<MethodsByRoute<R>, R>>) => void;
declare type Input<R extends Routes, M extends MethodsByRoute<R>> = {
    params: Params<R>;
    headers: RequestHeaders<M, R>;
    body: RequestBody<M, R>;
};
declare type Output<R extends Routes, M extends MethodsByRoute<R>> = {
    headers: ResponseHeaders<M, R>;
    body: ResponseBody<M, R>;
};
declare type MethodImplementation<R extends Routes, M extends MethodsByRoute<R>> = {
    headers: (headers: IncomingHttpHeaders) => headers is RequestHeaders<M, R>;
    body: (body: unknown) => body is RequestBody<M, R>;
    exec: (request: Input<R, M>) => Promise<Output<R, M>>;
};
declare type Methods<R extends Routes> = {
    [M in MethodsByRoute<R>]: MethodImplementation<R, M>;
};
export declare const makeHandler: <R extends never>(methods: Methods<R>) => Handler<R>;
